MODULE RTC;
(* =========================================================================  
   RTC - Maxim DS3234 Real-Time Clock SPI Driver
   
   Target: RISC5 Project Oberon

   (c) 2017 CFB Software   
   http://www.astrobe.com  

  Permission to use, copy, modify, and/or distribute this software and its
  accompanying documentation (the "Software") for any purpose with or
  without fee is hereby granted, provided that the above copyright notice
  and this permission notice appear in all copies.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
  WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY, FITNESS AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
  AUTHORS BE LIABLE FOR ANY CLAIM, SPECIAL, DIRECT, INDIRECT, OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES OR LIABILITY WHATSOEVER, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE DEALINGS IN OR USE OR PERFORMANCE OF THE SOFTWARE.
  ========================================================================= *)

IMPORT SPI := SPI1, SYSTEM;

CONST
  (* DS3234 Address Registers *)
  TimeAdr* = 00H;
  DateAdr* = 04H;
  Alarm1Adr* = 07H;
  Alarm2Adr* = 0BH;
  ControlAdr* = 0EH;
  StatusAdr* = 0FH;
  CrystalAdr* = 10H;
  TempAdr* = 11H;
  SRAMAdr* = 18H;
  SRAMDataAdr* = 19H;
  
  
PROCEDURE BCDToByte(bcd: BYTE): BYTE;
BEGIN
  RETURN (bcd DIV 16) * 10 + (bcd MOD 16)
END BCDToByte; 


PROCEDURE ByteToBCD(b: BYTE): BYTE;
BEGIN
  RETURN (b DIV 10) * 16 + (b MOD 10)
END ByteToBCD; 


PROCEDURE ReadReg*(adr: BYTE; VAR r: BYTE);
BEGIN
  SPI.Select(TRUE);
  SPI.SendByte(adr);
  SPI.ReceiveByte(r);
  SPI.Select(FALSE)
END ReadReg;


PROCEDURE WriteReg*(adr: BYTE; r: BYTE);
BEGIN
  SPI.Select(TRUE);
  SPI.SendByte(adr + 80H);
  SPI.SendByte(r);
  SPI.Select(FALSE)
END WriteReg;


PROCEDURE ReadRegs*(adr: BYTE; VAR r: ARRAY OF BYTE);
BEGIN
  SPI.Select(TRUE);
  SPI.SendByte(adr);
  SPI.Receive(r);
  SPI.Select(FALSE)
END ReadRegs; 
    

PROCEDURE WriteRegs*(adr: BYTE; r: ARRAY OF BYTE);
BEGIN
  SPI.Select(TRUE);
  SPI.SendByte(adr + 80H);
  SPI.Send(r);
  SPI.Select(FALSE)
END WriteRegs;
  
    
PROCEDURE ReadTime*(VAR hrs, mins, secs: BYTE);
VAR
  r: ARRAY 3 OF BYTE;
BEGIN
  ReadRegs(TimeAdr, r);
  secs := BCDToByte(r[0]);
  mins := BCDToByte(r[1]);
  hrs  := BCDToByte(r[2])
END ReadTime;


PROCEDURE WriteTime*(hrs, mins, secs: BYTE);
VAR
  r: ARRAY 3 OF BYTE;
BEGIN
  r[0] := ByteToBCD(secs);
  r[1] := ByteToBCD(mins);
  r[2] := ByteToBCD(hrs);
  WriteRegs(TimeAdr, r)
END WriteTime;


PROCEDURE ReadDate*(VAR dd, mm, yy: BYTE);
VAR
  r: ARRAY 3 OF BYTE;
BEGIN
  ReadRegs(DateAdr, r);
  dd   := BCDToByte(r[0]);
  mm   := BCDToByte(r[1]);
  yy   := BCDToByte(r[2])
END ReadDate;


PROCEDURE WriteDate*(dd, mm, yy: BYTE);
VAR
  r: ARRAY 3 OF BYTE;
BEGIN
  r[0] := ByteToBCD(dd);
  r[1] := ByteToBCD(mm);
  r[2] := ByteToBCD(yy);
  WriteRegs(DateAdr, r)
END WriteDate;


PROCEDURE SetClock*(dt: INTEGER);
VAR 
  dd, mm, yy, hrs, mins, secs: BYTE;
  r: ARRAY 7 OF BYTE;
BEGIN
  yy   := dt DIV 4000000H MOD 40H;
  mm   := dt DIV 400000H MOD 10H;
  dd   := dt DIV 20000H MOD 20H;   
  hrs  := dt DIV 1000H MOD 20H;
  mins := dt DIV 40H MOD 40H;
  secs := dt MOD 40H;
  r[0] := ByteToBCD(secs);
  r[1] := ByteToBCD(mins);
  r[2] := ByteToBCD(hrs);
  r[3] := 1;
  r[4] := ByteToBCD(dd);
  r[5] := ByteToBCD(mm);
  r[6] := ByteToBCD(yy);
  WriteRegs(TimeAdr, r)
END SetClock;  


PROCEDURE Clock*(): INTEGER;
VAR
  dd, mm, yy, hrs, mins, secs: BYTE;
  r: ARRAY 7 OF BYTE;
  dt: INTEGER;
BEGIN
  ReadRegs(TimeAdr, r);
  secs := BCDToByte(r[0]);
  mins := BCDToByte(r[1]);
  hrs  := BCDToByte(r[2]);
  dd   := BCDToByte(r[4]);
  mm   := BCDToByte(r[5]);
  yy   := BCDToByte(r[6]);
  dt := yy * 4000000H +
        mm * 400000H +
        dd * 20000H + 
        hrs * 1000H +
        mins * 40H +
        secs;
  RETURN dt
END Clock;


PROCEDURE Init*;
BEGIN
  SPI.Select(FALSE)
END Init;
  
END RTC.
